@@ -15,9 +15,13 @@
 
 JNIEnv* jniEnv;
 
+#if (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER >= 3)
+extern PGDLLIMPORT int log_min_messages;
+extern PGDLLIMPORT int client_min_messages;
+#else
 extern DLLIMPORT int log_min_messages;
 extern DLLIMPORT int client_min_messages;
-
+#endif
 static jobject s_threadLock;
 
 #define BEGIN_JAVA { JNIEnv* env = jniEnv; jniEnv = 0;
@@ -70,6 +74,16 @@ static void elogExceptionMessage(JNIEnv* env, jthrowable exh, int logLevel)
 	ereport(logLevel, (errcode(sqlState), errmsg(buf.data)));
 }
 
+static void printStacktrace(JNIEnv* env, jobject exh)
+{
+	if(DEBUG1 >= log_min_messages || DEBUG1 >= client_min_messages)
+	{
+		int currLevel = Backend_setJavaLogLevel(DEBUG1);
+		(*env)->CallVoidMethod(env, exh, Throwable_printStackTrace);
+		Backend_setJavaLogLevel(currLevel);
+	}
+}
+
 static void endCall(JNIEnv* env)
 {
 	jobject exh = (*env)->ExceptionOccurred(env);
@@ -82,13 +96,7 @@ static void endCall(JNIEnv* env)
 	jniEnv = env;
 	if(exh != 0)
 	{
-		if(DEBUG1 >= log_min_messages || DEBUG1 >= client_min_messages)
-		{
-			int currLevel = Backend_setJavaLogLevel(DEBUG1);
-			(*env)->CallVoidMethod(env, exh, Throwable_printStackTrace);
-			Backend_setJavaLogLevel(currLevel);
-		}
-
+		printStacktrace(env, exh);
 		if((*env)->IsInstanceOf(env, exh, ServerException_class))
 		{
 			/* Rethrow the server error.
@@ -504,7 +512,11 @@ void JNI_exceptionDescribe(void)
 	BEGIN_JAVA
 	exh = (*env)->ExceptionOccurred(env);
 	if(exh != 0)
+	{
+		(*env)->ExceptionClear(env);
+		printStacktrace(env, exh);
 		elogExceptionMessage(env, exh, WARNING);
+	}
 	END_JAVA
 }
 
