@@ -41,7 +41,16 @@ Java_org_postgresql_pljava_internal_Session__1setUser(JNIEnv* env, jclass cls, j
 	 * a finally block after an exception.
 	 */
 	BEGIN_NATIVE_NO_ERRCHECK
+#if ( \
+    (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER >= 3) \
+    || (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER == 2 && PGSQL_PATCH_VER >= 6)  \
+    || (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER == 1 && PGSQL_PATCH_VER >= 11) \
+    || (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER == 0 && PGSQL_PATCH_VER >= 15) \
+    )
+	SetUserIdAndContext(AclId_getAclId(aclId), true);
+#else
 	SetUserId(AclId_getAclId(aclId));
+#endif
 	END_NATIVE
 }
 
